#ifndef _PLUA_API_H
#define _PLUA_API_H 1

#define PLUA_API_VERSION	0x0110

// This is the Lua/Plua API available to libraries.
// The only way for a library to access Plua internals is by using
// these functions.

typedef enum {
  gadgetDraw, gadgetErase, gadgetDown, gadgetUp, gadgetMove, gadgetDelete,
  gadgetGetText, gadgetSetText, gadgetGetState, gadgetSetState
} PluaGadgetEvent;

typedef Boolean (*PluaGadgetCallback)(lua_State *L, PluaGadgetEvent event,
                                      int id, void *data, void *arg,
                                      RectangleType *rect);

#ifndef __va_list__
#define __va_list__
typedef char *va_list;
#endif

typedef struct {
  unsigned short version;

  // Plua extensions

  void (*plua_setlibdata)(char *name, void *data);
  void *(*plua_getlibdata)(char *name);
  int (*plua_pusherror)(lua_State *L, int err);

  // basic stack manipulation

  int (*lua_gettop)(lua_State *L);
  void (*lua_settop)(lua_State *L, int index);
  void (*lua_pushvalue)(lua_State *L, int index);
  void (*lua_remove)(lua_State *L, int index);
  void (*lua_insert)(lua_State *L, int index);
  int (*lua_stackspace)(lua_State *L);

  // access functions (stack -> C)

  int (*lua_type)(lua_State *L, int index);
  const char *(*lua_typename)(lua_State *L, int t);
  int (*lua_isnumber)(lua_State *L, int index);
  int (*lua_isstring)(lua_State *L, int index);
  int (*lua_iscfunction)(lua_State *L, int index);
  int (*lua_tag)(lua_State *L, int index);
  int (*lua_equal)(lua_State *L, int index1, int index2);
  int (*lua_lessthan)(lua_State *L, int index1, int index2);
  Number (*lua_tonumber)(lua_State *L, int index);
  const char *(*lua_tostring)(lua_State *L, int index);
  size_t (*lua_strlen)(lua_State *L, int index);
  lua_CFunction (*lua_tocfunction)(lua_State *L, int index);
  void *(*lua_touserdata)(lua_State *L, int index);
  const void *(*lua_topointer)(lua_State *L, int index);

  // push functions (C -> stack)

  void (*lua_pushnil)(lua_State *L);
  void (*lua_pushnumber)(lua_State *L, Number n);
  void (*lua_pushlstring)(lua_State *L, const char *s, size_t len);
  void (*lua_pushstring)(lua_State *L, const char *s);
  void (*lua_pushcclosure)(lua_State *L, lua_CFunction fn, int n);
  void (*lua_pushusertag)(lua_State *L, void *u, int tag);

  // get functions (Lua -> stack)

  void (*lua_gettable)(lua_State *L, int index);
  void (*lua_rawget)(lua_State *L, int index);
  void (*lua_rawgeti)(lua_State *L, int index, int n);
  void (*lua_newtable)(lua_State *L);
  void (*lua_getglobal)(lua_State *L, const char *name);
  void (*lua_getglobals)(lua_State *L);
  void (*lua_gettagmethod)(lua_State *L, int tag, const char *event);
  int  (*lua_getref)(lua_State *L, int ref);

  // set functions (stack -> Lua)

  void (*lua_settable)(lua_State *L, int index);
  void (*lua_rawset)(lua_State *L, int index);
  void (*lua_rawseti)(lua_State *L, int index, int n);
  void (*lua_setglobal)(lua_State *L, const char *name);
  void (*lua_setglobals)(lua_State *L);
  void (*lua_settagmethod)(lua_State *L, int tag, const char *event);
  int (*lua_ref)(lua_State *L, int lock);

  // "do" functions (run Lua code)

  int  (*lua_call)(lua_State *L, int nargs, int nresults);
  void (*lua_rawcall)(lua_State *L, int nargs, int nresults);
  int  (*lua_dofile)(lua_State *L, const char *filename);
  int  (*lua_dostring)(lua_State *L, const char *str);
  int  (*lua_dobuffer)(lua_State *L, const char *buff, size_t size, const char *name);

  // miscellaneous functions

  int  (*lua_newtag)(lua_State *L);
  int  (*lua_copytagmethods)(lua_State *L, int tagto, int tagfrom);
  void (*lua_settag)(lua_State *L, int tag);
  void (*lua_error)(lua_State *L, const char *s);
  void (*lua_errorx)(lua_State *L, const char *s, int errcode);
  void (*lua_unref)(lua_State *L, int ref);
  int  (*lua_next)(lua_State *L, int index);
  int  (*lua_getn)(lua_State *L, int index);
  void (*lua_concat)(lua_State *L, int n);
  void *(*lua_newuserdata)(lua_State *L, size_t size);

  // auxiliary funtions (lauxlib)

  void (*luaL_openlib)(lua_State *L, const struct luaL_reg *l, int n);
  void (*luaL_argerror)(lua_State *L, int numarg, const char *extramsg);
  const char *(*luaL_check_lstr)(lua_State *L, int numArg, size_t *len);
  const char *(*luaL_opt_lstr)(lua_State *L, int numArg, const char *def, size_t *len);
  Number (*luaL_check_number)(lua_State *L, int numArg);
  Number (*luaL_opt_number)(lua_State *L, int numArg, Number def);
  void (*luaL_checkstack)(lua_State *L, int space, const char *msg);
  void (*luaL_checktype)(lua_State *L, int narg, int t);
  void (*luaL_checkany)(lua_State *L, int narg);

  // libc funtions

  FILE *(*fopen)(const char *, const char *);
  int (*fclose)(FILE *);
  int (*feof)(FILE *);
  int (*fseek)(FILE *, long, int);
  long (*ftell)(FILE *);
  size_t (*fread)(void *, size_t, size_t, FILE *);
  size_t (*fwrite)(void *, size_t, size_t, FILE *);
  int (*fputs)(const char *, FILE *);
  char *(*fgets)(char *, int, FILE *);
  void *(*malloc)(size_t size);
  void *(*calloc)(size_t number, size_t size);
  void *(*realloc)(void *ptr, size_t size);
  void (*free)(void *ptr);

  // Plua internals

  void *(*plua_getwindow)(void);
  int (*plua_creategadget)(lua_State *L, int width, int height, void *data,
                           PluaGadgetCallback callback);
  void (*plua_pcolor)(long fg, long bg);
  long (*plua_prgb)(int r, int g, int b);
  void (*plua_prect)(int x, int y, int dx, int dy);
  void (*plua_pbox)(int x, int y, int dx, int dy);
  void (*plua_pcircle)(int x, int y, int rx, int ry);
  void (*plua_pdisc)(int x, int y, int rx, int ry);
  void (*plua_pline)(int x1, int y1, int x2, int y2);
  void (*plua_plineto)(int x, int y);
  void (*plua_pmoveto)(int x, int y);
  void (*plua_print)(char *s);
  void (*plua_ppos)(int *x, int *y);
  void (*plua_ptextdimensions)(char *s, int *w, int *h);
  void (*plua_getrgb)(long c, int *r, int *g, int *b);
  void (*plua_getcolor)(long *fg, long *bg);

} PluaAPI;

#endif
